using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Globalization;
using System.IO;
using System.Windows.Forms;
using System.Xml.Linq;

namespace MyDAMapLib {
    //Throws IOExceptions, Parsing Exceptions, NullArgumentExceptions - If DAData.xml is Incorrect
    public class MyDAMap {
        private BitArray sotpData = new BitArray(Convert.FromBase64String(XElement.Load("DAData.xml").Element("Data").Element("sotp").Value));

        public bool TryMakeDAMapRhombusImage(string mapFile, ushort mapDimX, ushort mapDimY, double scale, Color bgColor, PixelFormat pixelFormat, out Bitmap daMapImage) {
            ushort blockDimX = (ushort)(10 * scale), blockDimY = (ushort)(5 * scale);
            if(blockDimX < 2 || blockDimY < 1) { blockDimX = 2; blockDimY = 1; };

            int width = blockDimX * (mapDimX + 2 + mapDimY), height = blockDimY * (mapDimX + 2 + mapDimY);
            if(width * height > 15000000 && MessageBox.Show("Attempting to make a " + width.ToString(CultureInfo.InvariantCulture) + " by " + height.ToString(CultureInfo.InvariantCulture) + " dimention image...  Is this intended? Propabably Not! Suggestion:  Change Scale Value, or Change Dimention (If Rhombus).  Continue?", "Warning, Continue?", MessageBoxButtons.YesNo) == DialogResult.No) {
                daMapImage = null;
                return false;
            }

            daMapImage = new Bitmap(width, height, pixelFormat);
            using(Graphics daMapGraphics = Graphics.FromImage(daMapImage)) {
                using(SolidBrush sb = new SolidBrush(bgColor)) daMapGraphics.FillRectangle(sb, 0, 0, daMapImage.Width, daMapImage.Height);
                bool[,] daMap = GetMakeDAMapBool(mapFile, mapDimX, mapDimY);
                if(daMap == null) {
                    daMapImage = null;
                    return false;
                }

                Point[] currBlockDim;
                using(TextureBrush daFillTex = new TextureBrush(MyResources.Resources.texTRANS, System.Drawing.Drawing2D.WrapMode.Tile))
                    for(ushort i = 0; i < mapDimY; i++)
                        for(ushort c = 0; c < mapDimX; c++)
                            if(daMap[c, i]) {
                                currBlockDim = new Point[] { new Point(blockDimX * (mapDimY + 1 - i + c), blockDimY * (c + i)), new Point(blockDimX * (mapDimY + 2 - i + c), blockDimY * (c + i + 1)), new Point(blockDimX * (mapDimY + 1 - i + c), c * blockDimY + i * blockDimY + blockDimX), new Point(blockDimX * (mapDimY - i + c), blockDimY * (c + i + 1)) };
                                daMapGraphics.FillPolygon(daFillTex, currBlockDim);
                                if(c > 0 && !daMap[c - 1, i]) daMapGraphics.DrawLine(new Pen(Color.White), currBlockDim[3], currBlockDim[0]);
                                if(i > 0 && !daMap[c, i - 1]) daMapGraphics.DrawLine(new Pen(Color.White), currBlockDim[0], currBlockDim[1]);
                                if(c < mapDimX - 1 && !daMap[c + 1, i]) daMapGraphics.DrawLine(new Pen(Color.White), currBlockDim[1], currBlockDim[2]);
                                if(i < mapDimY - 1 && !daMap[c, i + 1]) daMapGraphics.DrawLine(new Pen(Color.White), currBlockDim[2], currBlockDim[3]);
                            }
            }
            return true;
        }
        public bool TryMakeDAMapSquareImage(string mapFilePath, ushort mapDimX, ushort mapDimY, double scale, Color bgColor, PixelFormat pixelFormat, out Bitmap daMapImage) {
            ushort blockDim = (ushort)(12 * scale);
            if(blockDim < 1) blockDim = 1;

            int width = mapDimX * blockDim, height = mapDimY * blockDim;
            if(width * height > 15000000 && MessageBox.Show("Attempting to make a " + width.ToString(CultureInfo.InvariantCulture) + " by " + height.ToString(CultureInfo.InvariantCulture) + " dimention image...  Is this intended? Propabably Not! Suggestion:  Change Scale Value, or Change Dimention (If Rhombus).  Continue?", "Warning, Continue?", MessageBoxButtons.YesNo) == DialogResult.No) {
                daMapImage = null;
                return false;
            }

            daMapImage = new Bitmap(width, height, pixelFormat);
            using(Graphics daMapGraphics = Graphics.FromImage(daMapImage)) {
                using(SolidBrush sb = new SolidBrush(bgColor)) daMapGraphics.FillRectangle(sb, 0, 0, daMapImage.Width, daMapImage.Height);
                bool[,] daMap = GetMakeDAMapBool(mapFilePath, mapDimX, mapDimY);
                if(daMap == null) {
                    daMapImage = null;
                    return false;
                }

                using(TextureBrush daFillTex = new TextureBrush(MyResources.Resources.texTRANS, System.Drawing.Drawing2D.WrapMode.Tile))
                    for(ushort i = 0; i < mapDimY; i++)
                        for(ushort c = 0; c < mapDimX; c++)
                            if(daMap[c, i]) {
                                daMapGraphics.FillRectangle(daFillTex, c * blockDim, i * blockDim, blockDim, blockDim);
                                if(c > 0 && !daMap[c - 1, i]) daMapGraphics.DrawLine(new Pen(Color.White), c * blockDim, i * blockDim, c * blockDim, (i + 1) * blockDim);
                                if(i > 0 && !daMap[c, i - 1]) daMapGraphics.DrawLine(new Pen(Color.White), c * blockDim, i * blockDim, (c + 1) * blockDim, i * blockDim);
                                if(c < mapDimX - 1 && !daMap[c + 1, i]) daMapGraphics.DrawLine(new Pen(Color.White), (c + 1) * blockDim, i * blockDim, (c + 1) * blockDim, (i + 1) * blockDim);
                                if(i < mapDimY - 1 && !daMap[c, i + 1]) daMapGraphics.DrawLine(new Pen(Color.White), c * blockDim, (i + 1) * blockDim, (c + 1) * blockDim, (i + 1) * blockDim);
                            }
            }
            return true;
        }

        //1 if Wall / 0 if No Wall
        public byte[,] GetMakeDAMapByte(string daMapFilePath, ushort dimX, ushort dimY) {
            FileInfo mapFile = new FileInfo(daMapFilePath);
            if(!mapFile.Exists || mapFile.Length / 6 != dimX * dimY) return null;

            byte[,] mainMap = new byte[dimX, dimY];
            using(BinaryReader daMapReader = new BinaryReader(new FileStream(daMapFilePath, FileMode.Open, FileAccess.Read, FileShare.ReadWrite)))
                for(ushort i = 0; i < dimY; i++)
                    for(ushort j = 0; j < dimX; j++) {
                        daMapReader.ReadUInt16();  //First Info in File is Tile (Not Walls)
                        if(isBlock(daMapReader.ReadUInt16(), daMapReader.ReadUInt16())) mainMap[j, i] = 1;
                    }
            return mainMap;
        }
        //'W' if Wall / (char)0 if No Wall
        public char[,] GetMakeDAMapChar(string daMapFilePath, ushort dimX, ushort dimY) {
            FileInfo mapFile = new FileInfo(daMapFilePath);
            if(!mapFile.Exists || mapFile.Length / 6 != dimX * dimY) return null;

            char[,] mainMap = new char[dimX, dimY];
            using(BinaryReader daMapReader = new BinaryReader(new FileStream(daMapFilePath, FileMode.Open, FileAccess.Read, FileShare.ReadWrite)))
                for(ushort i = 0; i < dimY; i++)
                    for(ushort j = 0; j < dimX; j++) {
                        daMapReader.ReadUInt16();  //First Info in File is Tile (Not Walls)
                        if(isBlock(daMapReader.ReadUInt16(), daMapReader.ReadUInt16())) mainMap[j, i] = 'W';
                    }
            return mainMap;
        }
        //True if Wall / False if No Wall
        public bool[,] GetMakeDAMapBool(string daMapFilePath, ushort dimX, ushort dimY) {
            FileInfo mapFile = new FileInfo(daMapFilePath);
            if(!mapFile.Exists || mapFile.Length / 6 != dimX * dimY) return null;

            bool[,] mainMap = new bool[dimX, dimY];
            using(BinaryReader daMapReader = new BinaryReader(new FileStream(daMapFilePath, FileMode.Open, FileAccess.Read, FileShare.ReadWrite)))
                for(ushort i = 0; i < dimY; i++)
                    for(ushort j = 0; j < dimX; j++) {
                        daMapReader.ReadUInt16();  //First Info in File is Tile (Not Walls)
                        if(isBlock(daMapReader.ReadUInt16(), daMapReader.ReadUInt16())) mainMap[j, i] = true;
                    }
            return mainMap;
        }
        private bool isBlock(ushort x, ushort y) {
            if(x == 0 && y == 0) return false;
            else if(x == 0) return sotpData[y - 1];
            else if(y == 0) return sotpData[x - 1];
            else return sotpData[x - 1] || sotpData[y - 1];
        }


        //X = 1/2*e.Y/blockDimY+1/2*e.X/blockDimX-1/2*mapDimY-1/2  X=Block# e.X=Position on Picture
        //Y = 1/2*e.Y/blockDimY-1/2*e.X/blockDimX+1/2*mapDimY+1/2  Y=Block# e.Y=Position on Picture
        public static bool TryColorBlockRhombus(int[] portals, ushort mapDimX, ushort mapDimY, Color fillColor, double scale, ref Bitmap daMapImage) {
            if(daMapImage == null) return false;

            ushort blockDimX = (ushort)(10 * scale), blockDimY = (ushort)(5 * scale);
            if(blockDimX < 2 || blockDimY < 1) { blockDimX = 2; blockDimY = 1; };

            using(Graphics daMapGraphics = Graphics.FromImage(daMapImage))
            using(SolidBrush sb = new SolidBrush(fillColor))
                for(byte i = 0; i < portals.Length - 1; i += 2) {
                    if(portals[i] > mapDimX || portals[i + 1] > mapDimY) return false;
                    daMapGraphics.FillPolygon(sb, new Point[] { new Point(blockDimX * (mapDimY + 1 - portals[i + 1] + portals[i]), blockDimY * (portals[i] + portals[i + 1])), new Point(blockDimX * (mapDimY + 2 - portals[i + 1] + portals[i]) - 1, blockDimY * (portals[i] + portals[i + 1] + 1)), new Point(blockDimX * (mapDimY + 1 - portals[i + 1] + portals[i]), portals[i] * blockDimY + portals[i + 1] * blockDimY + blockDimX - 1), new Point(blockDimX * (mapDimY - portals[i + 1] + portals[i]) + 1, blockDimY * (portals[i] + portals[i + 1] + 1)) });
                }
            return true;
        }
        //double X = 1.0 / 2.0 * e.Y / blockDimY + 1.0 / 2.0 * e.X / blockDimX - 1.0 / 2.0 * mapDimY - 1.0 / 2.0;  X=Block# e.X=Position on Picture
        //double Y = 1.0 / 2.0 * e.Y / blockDimY - 1.0 / 2.0 * e.X / blockDimX + 1.0 / 2.0 * mapDimY + 1.0 / 2.0;  Y=Block# e.Y=Position on Picture
        public static bool TryColorBlockSquare(int[] portals, Color fillColor, double scale, ref Bitmap daMapImage) {
            if(daMapImage == null) return false;

            ushort blockDim = (ushort)(12 * scale);
            if(blockDim < 1) blockDim = 1;

            using(Graphics daAMapGraphics = Graphics.FromImage(daMapImage))
            using(SolidBrush sb = new SolidBrush(fillColor))
                for(byte i = 0; i < portals.Length - 1; i += 2) {
                    if((portals[i] + 1) * blockDim > daMapImage.Width || (portals[i + 1] + 1) * blockDim > daMapImage.Height) return false;
                    daAMapGraphics.FillRectangle(sb, portals[i] * blockDim + 1, portals[i + 1] * blockDim + 1, blockDim - 1, blockDim - 1);
                }
            return true;
        }

        public static List<ushort[]> GetFactors(string mapFilePath) {
            FileInfo mapInfo = new FileInfo(@mapFilePath);
            if(!mapInfo.Exists) return null;

            //Finds the Factors
            int mapLength = (int)(mapInfo.Length / 6), divRemainder, divResult;
            double maxLength = Math.Sqrt(mapLength);

            List<ushort[]> myFactors = new List<ushort[]>();
            for(byte i = 1; i <= maxLength; i++) {
                divResult = Math.DivRem(mapLength, i, out divRemainder);
                if(divRemainder == 0) myFactors.Add(new ushort[] { (ushort)i, (ushort)divResult });
            }
            foreach(ushort[] A in myFactors.ToArray()) if(A[0] != A[1]) myFactors.Add(new ushort[] { A[1], A[0] });
            myFactors.Reverse((int)Math.Ceiling((double)myFactors.Count / 2), myFactors.Count / 2);

            return myFactors;
        }
    }
}