using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;


namespace DATabExtractor {
    public partial class DATabExtractorForm : Form {
        private Bitmap DAMapImage;
        private List<ushort[]> myFactors = new List<ushort[]>();
        private MyDAMapLib.MyDAMap myDAMap;


        public DATabExtractorForm() {
            InitializeComponent();
            this.Text = "DATabExtractor v" + Application.ProductVersion;

            if(!System.IO.File.Exists("DAData.xml")) { MessageBox.Show("DAData.xml file not found: Download the file from www.da-wizard.com."); Environment.Exit(0); }
            myDAMap = new MyDAMapLib.MyDAMap();

            DisplayAsComboBox.SelectedIndex = 0;
            ImageFormatComboBox.DataSource = new object[] { System.Drawing.Imaging.ImageFormat.Bmp, System.Drawing.Imaging.ImageFormat.Gif, System.Drawing.Imaging.ImageFormat.Jpeg, System.Drawing.Imaging.ImageFormat.Png, System.Drawing.Imaging.ImageFormat.Tiff };
            ImageFormatComboBox.SelectedItem = System.Drawing.Imaging.ImageFormat.Gif;
            //PixelFormatComboBox.DataSource = Enum.GetValues(typeof(System.Drawing.Imaging.PixelFormat));  //Only 3 formates really supported...
            PixelFormatComboBox.DataSource = new object[] { System.Drawing.Imaging.PixelFormat.Format16bppRgb555, System.Drawing.Imaging.PixelFormat.Format32bppArgb, System.Drawing.Imaging.PixelFormat.Format32bppRgb };
            PixelFormatComboBox.SelectedItem = System.Drawing.Imaging.PixelFormat.Format16bppRgb555;

            List<object> ALColor = new List<object>();
            foreach(object clr in Enum.GetValues(typeof(KnownColor))) if(!Color.FromKnownColor((KnownColor)clr).IsSystemColor) ALColor.Add(clr);
            BGColorComboBox.DataSource = ALColor;
            BGColorComboBox.SelectedItem = KnownColor.Black;
        }

        private void OpenDAMapFileTextBox_KeyPress(object sender, KeyPressEventArgs e) {
            if(e.KeyChar != (char)Keys.Enter) return;
            MapDimList.Items.Clear();
            if((myFactors = MyDAMapLib.MyDAMap.GetFactors(OpenDAMapFileTextBox.Text)) == null) {
                DAMapImage = null;
                ResetPreivewIfChecked();
                MessageBox.Show("File doesn't Exsist");
                return;
            }

            string defaultPath = "";
            foreach(string A in SaveMapTextBox.Text.Split('\\')) if(A != "" && !A.Contains(".")) defaultPath += A + "\\";
            string[] name = OpenDAMapFileTextBox.Text.Split('.')[0].Split('\\');
            SaveMapTextBox.Text = defaultPath + name[name.Length - 1] + "." + ImageFormatComboBox.Text.ToLower();

            foreach(ushort[] A in myFactors) MapDimList.Items.Add(A[0] + "x" + A[1]);
            MapDimList.SelectedIndex = myFactors.Count / 2;
        }
        private void DAMapFileBrowse_Click(object sender, EventArgs e) {
            OpenFileDialog DAMapFileBrowse = new OpenFileDialog();
            DAMapFileBrowse.Title = "Open DA Map File";
            DAMapFileBrowse.Filter = "DA Map Files (*.map)|*.map";
            DAMapFileBrowse.InitialDirectory = Application.StartupPath;
            DAMapFileBrowse.RestoreDirectory = true;
            if(DAMapFileBrowse.ShowDialog() == DialogResult.OK) {
                OpenDAMapFileTextBox.Text = DAMapFileBrowse.FileName;
                OpenDAMapFileTextBox_KeyPress(null, new KeyPressEventArgs((char)Keys.Enter));
            }
        }

        private void SaveMapTextBox_KeyPress(object sender, KeyPressEventArgs e) {
            if(e.KeyChar != (char)Keys.Enter) return;
            if(SaveMapTextBox.Text == "") { MessageBox.Show("Invalid Path, Please Give a No Empty Save Path"); return; } else if(DAMapImage == null) { MessageBox.Show("Nothing to Save, Please Load a Map"); return; }

            DAMapImage.Save(@SaveMapTextBox.Text, (System.Drawing.Imaging.ImageFormat)ImageFormatComboBox.SelectedItem);
            MessageBox.Show("Save Successfully to \"" + SaveMapTextBox.Text + "\"");
        }
        private void SaveOutputFileBrowse_Click(object sender, EventArgs e) {
            SaveFileDialog DAMapSaveBrowse = new SaveFileDialog();
            DAMapSaveBrowse.Title = "Save Tab Map File";
            DAMapSaveBrowse.Filter = "Graphic File (*." + ImageFormatComboBox.Text + ")|*." + ImageFormatComboBox.Text.ToLower();
            DAMapSaveBrowse.InitialDirectory = Application.StartupPath;
            DAMapSaveBrowse.RestoreDirectory = true;
            if(DAMapSaveBrowse.ShowDialog() == DialogResult.OK) {
                SaveMapTextBox.Text = DAMapSaveBrowse.FileName;
                SaveMapTextBox_KeyPress(null, new KeyPressEventArgs((char)Keys.Enter));
            }
        }

        private void UnloadButton_Click(object sender, EventArgs e) {
            if(DAMapImage == null) return;
            else { myFactors.Clear(); MapDimList.Items.Clear(); DAMapImage = null; ResetPreivewIfChecked(); }
        }

        private void FullPreviewButton_Click(object sender, EventArgs e) {
            if(DAMapImage != null) new FullImagePreview(DAMapImage, Color.FromKnownColor((KnownColor)BGColorComboBox.SelectedItem)).Show();
            else MessageBox.Show("Nothing to View... Please Load Map");
        }

        private void PixelFormatComboBox_SelectedIndexChanged(object sender, EventArgs e) { if(DAMapImage != null) OpenDAMapFileTextBox_KeyPress(null, new KeyPressEventArgs((char)Keys.Enter)); }

        private void ImageFormatComboBox_SelectedIndexChanged(object sender, EventArgs e) { if(SaveMapTextBox.Text != "") SaveMapTextBox.Text = SaveMapTextBox.Text.Split('.')[0] + "." + ImageFormatComboBox.Text.ToLower(); }

        private void MapDimList_SelectedIndexChanged(object sender, EventArgs e) { CreateDABitMap(); }

        private void DisplayAsComboBox_SelectedIndexChanged(object sender, EventArgs e) { if(DAMapImage != null) CreateDABitMap(); }

        private void BGColorComboBox_SelectedIndexChanged(object sender, EventArgs e) { if(DAMapImage != null) CreateDABitMap(); }

        private void ImageScaleTextBox_KeyPress(object sender, KeyPressEventArgs e) { if(e.KeyChar == (char)Keys.Enter && DAMapImage != null) CreateDABitMap(); }

        private void QuickPreviewCheckBox_CheckedChanged(object sender, EventArgs e) { ResetPreivewIfChecked(); }

        private void HelpLable_Click(object sender, EventArgs e) { new HelpAboutBox().Show(); }

        private void ResetPreivewIfChecked() {
            if(QuickPreviewCheckBox.Checked && DAMapImage != null) {
                QuickPreviewPicBox.BackColor = Color.FromKnownColor((KnownColor)BGColorComboBox.SelectedItem);
                if(DisplayAsComboBox.SelectedIndex == 0) QuickPreviewPicBox.Image = DAMapImage.GetThumbnailImage(380, 190, null, IntPtr.Zero);
                else {
                    if(DAMapImage.Height >= DAMapImage.Width) QuickPreviewPicBox.Image = DAMapImage.GetThumbnailImage((int)Math.Ceiling((double)190 * DAMapImage.Width / DAMapImage.Height), 190, null, IntPtr.Zero);
                    else QuickPreviewPicBox.Image = DAMapImage.GetThumbnailImage(380, (int)Math.Ceiling((double)380 * DAMapImage.Height / DAMapImage.Width), null, IntPtr.Zero);
                }
            } else if(QuickPreviewPicBox.Image != null) {
                QuickPreviewPicBox.Image = null;
                QuickPreviewPicBox.BackColor = Color.Transparent;
            }
        }


        private void CreateDABitMap() {
            double result;
            if(!double.TryParse(ImageScaleTextBox.Text, out result)) {
                MessageBox.Show("Scale is not a Number.");
                return;
            }

            LoadingLable.Enabled = true;
            if(DisplayAsComboBox.SelectedIndex == 0) {
                if(!myDAMap.TryMakeDAMapRhombusImage(OpenDAMapFileTextBox.Text, myFactors[MapDimList.SelectedIndex][0], myFactors[MapDimList.SelectedIndex][1], result / 100, Color.FromKnownColor((KnownColor)BGColorComboBox.SelectedItem), (System.Drawing.Imaging.PixelFormat)PixelFormatComboBox.SelectedItem, out DAMapImage)) UnloadButton_Click(null, null);
            } else if(!myDAMap.TryMakeDAMapSquareImage(OpenDAMapFileTextBox.Text, myFactors[MapDimList.SelectedIndex][0], myFactors[MapDimList.SelectedIndex][1], result / 100, Color.FromKnownColor((KnownColor)BGColorComboBox.SelectedItem), (System.Drawing.Imaging.PixelFormat)PixelFormatComboBox.SelectedItem, out DAMapImage)) UnloadButton_Click(null, null);

            ResetPreivewIfChecked();
            LoadingLable.Enabled = false;
        }
    }
}